@@ -101,7 +101,7 @@ testmap = [
     RegrTest('test_c_locale_coercion.py'),
     RegrTest('test_calendar.py'),
     RegrTest('test_call.py'),
-    RegrTest('test_capi.py'),
+    RegrTest('test_capi'),
     RegrTest('test_cgi.py'),
     RegrTest('test_cgitb.py'),
     RegrTest('test_charmapcodec.py'),
@@ -220,7 +220,6 @@ testmap = [
     RegrTest('test_genericclass.py'),
     RegrTest('test_genericpath.py'),
     RegrTest('test_genexps.py'),
-    RegrTest('test_getargs2.py'),
     RegrTest('test_getopt.py'),
     RegrTest('test_getpass.py'),
     RegrTest('test_gettext.py'),
@@ -248,6 +247,7 @@ testmap = [
     RegrTest('test_importlib'),
     RegrTest('test_index.py'),
     RegrTest('test_inspect.py'),
+    RegrTest('test_interpreters.py'),
     RegrTest('test_int.py'),
     RegrTest('test_int_literal.py'),
     RegrTest('test_io.py'),
@@ -296,6 +296,7 @@ testmap = [
     RegrTest('test_nntplib.py'),
     RegrTest('test_ntpath.py'),
     RegrTest('test_numeric_tower.py'),
+    RegrTest('test_opcache.py'),
     RegrTest('test_opcodes.py'),
     RegrTest('test_openpty.py'),
     RegrTest('test_operator.py'),
@@ -304,11 +305,10 @@ testmap = [
     RegrTest('test_os.py'),
     RegrTest('test_ossaudiodev.py'),
     RegrTest('test_osx_env.py'),
-    RegrTest('test_parser.py', skip="slowly deprecating compiler"),
     RegrTest('test_pathlib.py'),
+    RegrTest('test_patma.py'),
     RegrTest('test_pdb.py'),
     RegrTest('test_peepholer.py'),
-    RegrTest('test_peg_parser.py'),
     RegrTest('test_peg_generator'),
     RegrTest('test_pickle.py'),
     RegrTest('test_picklebuffer.py'),
@@ -388,7 +388,6 @@ testmap = [
     RegrTest('test_strptime.py'),
     RegrTest('test_strtod.py'),
     RegrTest('test_struct.py'),
-    RegrTest('test_structmembers.py'), # skip="CPython specific"),
     RegrTest('test_structseq.py'),
     RegrTest('test_subclassinit.py'),
     RegrTest('test_subprocess.py'),
@@ -396,7 +395,6 @@ testmap = [
     RegrTest('test_sundry.py'),
     RegrTest('test_super.py'),
     RegrTest('test_support.py'),
-    RegrTest('test_symbol.py'),
     RegrTest('test_symtable.py', skip="implementation detail"),
     RegrTest('test_syntax.py'),
     RegrTest('test_sys.py'),
@@ -433,6 +431,7 @@ testmap = [
     RegrTest('test_type_comments.py'),
     RegrTest('test_typechecks.py'),
     RegrTest('test_types.py'),
+    RegrTest('test_type_annotations.py'),
     RegrTest('test_typing.py'),
     RegrTest('test_ucn.py'),
     RegrTest('test_unary.py'),
@@ -480,6 +479,7 @@ testmap = [
     RegrTest('test_xmlrpc.py'),
     RegrTest('test_xmlrpc_net.py'),
     RegrTest('test_xxtestfuzz.py', skip="CPython internal details"),
+    RegrTest('test_xxlimited.py', skip="CPython internal details"),
     RegrTest('test_yield_from.py'),
     RegrTest('test_zipapp.py'),
     RegrTest('test_zipfile.py'),
